/***************************************************************************
 * LPRng - An Extended Print Spooler System
 *
 * Copyright 1988-2000, Patrick Powell, San Diego, CA
 *     papowell@astart.com
 * See LICENSE for conditions of use.
 *
 ***************************************************************************/

 static char *const _id =
"$Id: sendmail.c,v 5.8 2000/05/31 00:35:22 papowell Exp papowell $";

#include "lp.h"
#include "errorcodes.h"
#include "fileopen.h"
#include "getqueue.h"
#include "sendmail.h"
#include "child.h"
/**** ENDINCLUDE ****/

/*
 * sendmail --- tell people about job completion
 * 1. fork a sendmail process
 * 2. if successful, send the good news
 * 3. if unsuccessful, send the bad news
 */

void Sendmail_to_user( int retval, struct job *job )
{
	char buffer[SMALLBUFFER];
	int n, tempfd;
	char *id, *mailname, *opname, *s;

	/*
	 * check to see if the user really wanted
	 * "your file was printed ok" message
	 */
	id = Find_str_value(&job->info,IDENTIFIER,Value_sep);
	if(!id) id = Find_str_value(&job->info,TRANSFERNAME,Value_sep);
	mailname = Find_str_value(&job->info,MAILNAME,Value_sep);
	opname = Mail_operator_on_error_DYN;
	DEBUG2("Sendmail_to_user: user '%s', operator '%s', sendmail '%s'",
		mailname, opname, Sendmail_DYN );

	if( !safestrchr( mailname,'%') && !safestrchr(mailname,'&') ) mailname = 0;
	if( !safestrchr( opname,'%') && !safestrchr(opname,'&') ) opname = 0;
	if( retval == JSUCC ) opname = 0;
	if( Sendmail_DYN == 0 ) return;
	if( mailname == 0 && opname == 0 ) return;

	tempfd = Make_temp_fd( 0 ); 

	DEBUG2("Sendmail_to_user: using '%s'", mailname );
	

	if( mailname ){
		SNPRINTF( buffer, sizeof(buffer), "To: %s\n", mailname );
		if( Write_fd_str( tempfd, buffer ) < 0 ) goto wr_error;
	}
	if( opname ){
		SNPRINTF(buffer,sizeof(buffer),
		"%s: %s\n", mailname?"CC":"To", opname );
		if( Write_fd_str( tempfd, buffer ) < 0 ) goto wr_error;
	}
	SNPRINTF(buffer,sizeof(buffer),
		"From: %s@%s\n",
		Mail_from_DYN ? Mail_from_DYN : Printer_DYN, FQDNHost_FQDN );
	if( Write_fd_str( tempfd, buffer ) < 0 ) goto wr_error;

	SNPRINTF(buffer,sizeof(buffer),
		"Subject: %s@%s job %s\n\n",
		Printer_DYN, FQDNHost_FQDN, id );
	if( Write_fd_str( tempfd, buffer ) < 0 ) goto wr_error;

	/* now do the message */
	SNPRINTF(buffer,sizeof(buffer),
		_("printer %s job %s"), Printer_DYN, id );
	if( Write_fd_str( tempfd, buffer ) < 0 ) goto wr_error;

	switch( retval) {
	case JSUCC:
		SNPRINTF(buffer,sizeof(buffer),
		_(" was successful.\n"));
		break;

	case JFAIL:
		SNPRINTF(buffer,sizeof(buffer),
		_(" failed, and retry count was exceeded.\n") );
		break;

	case JABORT:
		SNPRINTF(buffer,sizeof(buffer),
		_(" failed and could not be retried.\n") );
		break;

	default:
		SNPRINTF(buffer,sizeof(buffer),
		_(" died a horrible death.\n"));
		break;
	}
	if( Write_fd_str( tempfd, buffer ) < 0 ) goto wr_error;

	/*
	 * get the last status of the spooler
	 */
	if( (s = Get_file_image( Spool_dir_DYN, Queue_status_file_DYN, Max_status_size_DYN )) ){
		if( Write_fd_str( tempfd, "\nStatus:\n\n" ) ||
			Write_fd_str( tempfd, s ) ) goto wr_error;
		if(s) free(s); s = 0;
	}

	if( Status_file_DYN && (s = Get_file_image( Spool_dir_DYN,
		Status_file_DYN, Max_status_size_DYN )) ){
		if( Write_fd_str( tempfd, "\nFilter Status:\n\n" ) ||
			Write_fd_str( tempfd, s ) ) goto wr_error;
		if(s) free(s); s = 0;
	}
	if( lseek( tempfd, 0, SEEK_SET ) == -1 ){
		Errorcode = JABORT;
		LOGERR_DIE(LOG_ERR) "Sendmail_to_user: seek failed");
	}
	n = Filter_file( tempfd, -1, "MAIL", Sendmail_DYN, 0, job, 0 );
	if( n ){
		Errorcode = JABORT;
		LOGERR(LOG_ERR) "Sendmail_to_user: sendmail failed '%s'", Server_status(n) );
	}
	return;

 wr_error:
	Errorcode = JABORT;
	LOGERR_DIE(LOG_ERR) "Sendmail_to_user: write failed");
}
